@@ -1,6 +1,6 @@
 /* client.c
  *
- * $Id: client.c,v 1.122 2001/08/12 20:21:43 nisse Exp $ */
+ * $Id: client.c,v 1.129 2001/09/04 13:23:23 nisse Exp $ */
 
 /* lsh, an implementation of the ssh protocol
  *
@@ -169,7 +169,6 @@ make_request_service(int service)
 static void
 do_exit_status(struct channel_request *c,
 	       struct ssh_channel *channel,
-	       struct ssh_connection *connection UNUSED,
 	       struct channel_request_info *info,
 	       struct simple_buffer *args,
 	       struct command_continuation *s,
@@ -200,7 +199,6 @@ do_exit_status(struct channel_request *c,
 static void
 do_exit_signal(struct channel_request *c,
 	       struct ssh_channel *channel,
-	       struct ssh_connection *connection UNUSED,
 	       struct channel_request_info *info,
 	       struct simple_buffer *args,
 	       struct command_continuation *s,
@@ -286,7 +284,7 @@ new_session(struct channel_open_command *s,
   CAST(session_open_command, self, s);
   struct ssh_channel *res;
 
-  self->session->write = connection->write;
+  self->session->connection = connection;
   
   *request = format_channel_open(ATOM_SESSION,
 				 local_channel_number,
@@ -385,10 +383,13 @@ make_client_session(struct client_options *options);
 void
 init_client_options(struct client_options *self,
 		    struct io_backend *backend,
+		    struct randomness_with_poll *random,
 		    struct exception_handler *handler,
 		    int *exit_code)			 
 {
   self->backend = backend;
+  self->random = random;
+  
   self->tty = make_unix_interact(backend);
   self->escape = -1;
   
@@ -404,7 +405,8 @@ init_client_options(struct client_options *self,
 
   self->with_remote_peers = 0; 
   self->with_pty = -1;
-
+  self->with_x11 = 0;
+    
   self->stdin_file = NULL;
   self->stdout_file = NULL;
   self->stderr_file = NULL;
@@ -415,7 +417,8 @@ init_client_options(struct client_options *self,
 
   self->used_stdin = 0;
   self->used_pty = 0;
-
+  self->used_x11 = 0;
+  
   self->start_shell = 1;
   self->remote_forward = 0;
 
@@ -492,6 +495,11 @@ client_options[] =
     "is applied to stderr only.", 0 },
   { "no-cvs-workaround", OPT_FORK_STDIO | ARG_NOT, NULL, 0,
     "Disable the cvs workaround.", 0 },
+
+  /* FIXME: Perhaps this should be moved from client.c to lsh.c? It
+   * doesn't work with lshg. Or perhaps that can be fixed?
+   * About the same problem applies to -R. */
+
 #if WITH_PTY_SUPPORT
   { "pty", 't', NULL, 0, "Request a remote pty (default).", 0 },
   { "no-pty", 't' | ARG_NOT, NULL, 0, "Don't request a remote pty.", 0 },
@@ -532,19 +540,10 @@ make_client_start_session(struct command *request)
   return r;
 }
 
-/* Create an interactive session */
-static struct command *
-client_shell_session(struct client_options *options)
+static void
+client_maybe_pty(struct client_options *options,
+		 struct object_queue *q)
 {
-  struct command *get_pty = NULL;
-  struct command *get_shell;
-  
-  struct object_list *session_requests;
-  struct ssh_channel *session = make_client_session(options);
-
-  if (!session)
-    return NULL;
-  
 #if WITH_PTY_SUPPORT
   if (options->with_pty && !options->used_pty)
     {
@@ -552,40 +551,75 @@ client_shell_session(struct client_options *options)
       
       if (options->tty && INTERACT_IS_TTY(options->tty))
 	{
-	  get_pty = make_pty_request(options->tty);
-	  if (!get_pty)
-	    {
-	      werror("lsh: Can't use tty (probably getattr or atexit failed).\n");
-	    }
+	  struct command *get_pty = make_pty_request(options->tty);
+
+	  if (get_pty)
+	    object_queue_add_tail(q,
+				  /* Ignore EXC_CHANNEL_REQUEST for the pty allocation call. */
+				  &make_catch_apply
+				  (make_catch_handler_info(EXC_ALL, EXC_CHANNEL_REQUEST,
+							   0, NULL),
+				   get_pty)->super);
+	  else
+	    werror("lsh: Can't use tty (probably getattr or atexit failed).\n");
 	}
       else
-	{
-	  werror("lsh: No tty available.\n");
-	}
+	werror("lsh: No tty available.\n");
     }
+#endif
+}
 
-  get_shell = make_client_start_session(&request_shell.super);
+static void
+client_maybe_x11(struct client_options *options,
+		 struct object_queue *q)
+{
+  if (options->with_x11)
+    {
+      char *display = getenv("DISPLAY");
+      struct command *request = NULL;
+      
+      assert(options->random);
+      if (display)
+	request = make_forward_x11(display, &options->random->super);
+	  
+      if (request)
+	{
+	  object_queue_add_tail(q, &request->super);
+	  options->used_x11 = 1;
+	}
+      else
+	werror("Can't find any local X11 display to forward.\n");
+    }
+}
+
+/* Create an interactive session */
+static struct command *
+client_shell_session(struct client_options *options)
+{
+  struct ssh_channel *session = make_client_session(options);
   
-  /* FIXME: We need a non-varargs constructor for lists. */
-  if (get_pty)
-    session_requests
-      = make_object_list(2,
-			 /* Ignore EXC_CHANNEL_REQUEST for the pty allocation call. */
-			 make_catch_apply
-			 (make_catch_handler_info(EXC_ALL, EXC_CHANNEL_REQUEST,
-						  0, NULL),
-			  get_pty),
-			 get_shell, -1);
+  if (session)
+    {
+      struct object_queue session_requests;
+    
+      object_queue_init(&session_requests);
+  
+      client_maybe_pty(options, &session_requests);
+      client_maybe_x11(options, &session_requests);
+  
+      object_queue_add_tail(&session_requests,
+			    &make_client_start_session(&request_shell.super)->super);
+  
+      {
+	CAST_SUBTYPE(command, r,
+		     make_start_session
+		     (make_open_session_command(session),
+		      queue_to_list_and_kill(&session_requests)));
+	return r;
+      }
+    }
   else
-#endif /* WITH_PTY_SUPPORT */
-    session_requests = make_object_list(1, get_shell, -1);
-
-  {
-    CAST_SUBTYPE(command, r,
-		 make_start_session
-		 (make_open_session_command(session), session_requests));
-    return r;
-  }
+    return NULL;
 }
 
 /* Create a session executing a command line */
@@ -594,16 +628,28 @@ client_command_session(struct client_options *options,
 		       struct lsh_string *command)
 {
   struct ssh_channel *session = make_client_session(options);
-
+  
   if (session)
     {
-      CAST_SUBTYPE(command, r,
-		   make_start_session
-		   (make_open_session_command(session),
-		    make_object_list
-		    (1, make_client_start_session(make_exec_request(command)),
-		     -1)));
-      return r;
+      struct object_queue session_requests;
+    
+      object_queue_init(&session_requests);
+  
+      /* NOTE: Doesn't ask for a pty. That's traditional behaviour,
+       * although perhaps not the Right Thing. */
+      
+      client_maybe_pty(options, &session_requests);
+      client_maybe_x11(options, &session_requests);
+
+      object_queue_add_tail(&session_requests,
+			    &make_client_start_session(make_exec_request(command))->super);
+      {
+	CAST_SUBTYPE(command, r,
+		     make_start_session
+		     (make_open_session_command(session),
+		      queue_to_list_and_kill(&session_requests)));
+	return r;
+      }
     }
   
   return NULL;
@@ -707,16 +753,40 @@ fork_output(int out)
     }
 }
 
+/* FIXME: Use const? */
+
+#define DEFINE_CALLBACK(name) \
+static void do_##name(struct lsh_callback *self); \
+static struct lsh_callback \
+name = { STATIC_HEADER, do_##name }; \
+static void do_##name(struct lsh_callback *self UNUSED)
+
 /* A callback that exits the process immediately. */
-static void
-do_exit(struct lsh_callback *self UNUSED)
+DEFINE_CALLBACK(exit_callback)
 {
   exit(EXIT_SUCCESS);
 }
 
-/* FIXME: Use const? */
-static struct lsh_callback
-exit_callback = { STATIC_HEADER, do_exit };
+DEFINE_CALLBACK(verbose_callback)
+{
+  verbose_flag = !verbose_flag;
+  if (verbose_flag)
+    verbose("Enabling verbose messages\n");
+}
+
+DEFINE_CALLBACK(debug_callback)
+{
+  debug_flag = !debug_flag;
+  if (debug_flag)
+    debug("Enabling debug messages\n");
+}
+
+DEFINE_CALLBACK(quiet_callback)
+{
+  quiet_flag = !quiet_flag;
+  if (!quiet_flag)
+    werror("Enabling warning messages\n");
+}
 
 DEFINE_COMMAND(background_process)
      (struct command *s UNUSED,
@@ -802,6 +872,11 @@ make_client_session(struct client_options *options)
       /* Bind ^Z to suspend. */
       escape->dispatch[26] = &suspend_callback;
       escape->dispatch['.'] = &exit_callback;
+
+      /* Toggle the verbosity flags */
+      escape->dispatch['d'] = &debug_callback;
+      escape->dispatch['v'] = &verbose_callback;
+      escape->dispatch['q'] = &quiet_callback;      
     }
   
   debug("lsh.c: Setting up stdout\n");
@@ -979,11 +1054,17 @@ client_argp_parser(int key, char *arg, struct argp_state *state)
 			  make_install_fix_channel_open_handler
 			  (ATOM_FORWARDED_TCPIP, &channel_open_forwarded_tcpip));
 #endif /* WITH_TCP_FORWARD */
-      
+
       /* Add shell action */
       if (options->start_shell)
 	client_add_action(options, client_shell_session(options));
 
+      if (options->used_x11)
+	client_add_action(options,
+			  make_install_fix_channel_open_handler
+			  (ATOM_X11,
+			   make_channel_open_x11(options->backend)));
+      
       /* Install suspend-handler */
       suspend_install_handler();
       break;
@@ -1051,7 +1132,7 @@ client_argp_parser(int key, char *arg, struct argp_state *state)
 	options->remote_forward = 1;
 	break;
       }      
-#endif 
+#endif
     case 'N':
       options->start_shell = 0;
       break;
@@ -1066,7 +1147,7 @@ client_argp_parser(int key, char *arg, struct argp_state *state)
 #if WITH_PTY_SUPPORT
     CASE_FLAG('t', with_pty);
 #endif /* WITH_PTY_SUPPORT */
-
+    
     CASE_ARG(OPT_STDIN, stdin_file, "/dev/null");
     CASE_ARG(OPT_STDOUT, stdout_file, "/dev/null"); 
     CASE_ARG(OPT_STDERR, stderr_file, "/dev/null");
