@@ -1,6 +1,6 @@
 /* io_commands.c
  *
- * $Id: io_commands.c,v 1.40 2001/04/08 19:00:10 nisse Exp $
+ * $Id: io_commands.c,v 1.44 2001/07/06 21:13:27 nisse Exp $
  */
 
 /* lsh, an implementation of the ssh protocol
@@ -44,6 +44,7 @@
 
 #include "io_commands.c.x"
 
+/* Used only by lsh-writekey */
 /* GABA:
    (class
      (name backend_command)
@@ -79,7 +80,11 @@ do_io_write_file(struct command *s,
     EXCEPTION_RAISE(e, make_io_exception(EXC_IO_OPEN_WRITE, NULL, errno, NULL));
 }
 
-DEFINE_COMMAND_SIMPLE(io_write_file_command, a)
+DEFINE_COMMAND(io_write_file_command)
+     (struct command *s UNUSED,
+      struct lsh_object *a,
+      struct command_continuation *c,
+      struct exception_handler *e UNUSED)
 {
   CAST(io_backend, backend, a);
 
@@ -87,7 +92,7 @@ DEFINE_COMMAND_SIMPLE(io_write_file_command, a)
   self->super.call = do_io_write_file;
   self->backend = backend;
 
-  return &self->super.super;
+  COMMAND_RETURN(c, self);
 }
 
 struct io_write_file_info *
@@ -164,61 +169,24 @@ do_listen(struct io_backend *backend,
  * Suitable for handling forwarding requests. NOTE: The calling
  * function has to do all remembering of the fd:s. */
 
-/* GABA:
-   (class
-     (name listen_with_callback)
-     (super command)
-     (vars
-       (callback object command)
-       (backend object io_backend)))
-*/
-
-static void
-do_listen_with_callback(struct command *s,
-			struct lsh_object *x,
-			struct command_continuation *c,
-			struct exception_handler *e)
+DEFINE_COMMAND3(listen_with_callback)
+     (struct lsh_object *a1,
+      struct lsh_object *a2,
+      struct lsh_object *a3,
+      struct command_continuation *c,
+      struct exception_handler *e)
 {
-  CAST(listen_with_callback, self, s);
-  CAST(address_info, address, x);
+  CAST_SUBTYPE(command, callback, a1);
+  CAST(io_backend, backend, a2);
+  CAST(address_info, address, a3);
 
   /* No dns lookups */
-  do_listen(self->backend, address,
+  do_listen(backend, address,
 	    c,
-	    make_apply(self->callback,
+	    make_apply(callback,
 		       &discard_continuation, e), e);
 }
 
-struct command *
-make_listen_with_callback(struct command *callback,
-			  struct io_backend *backend)
-{
-  NEW(listen_with_callback, self);
-  self->callback = callback;
-  self->backend = backend;
-
-  self->super.call = do_listen_with_callback;
-
-  return &self->super;
-}
-
-static struct lsh_object *
-collect_listen_callback(struct collect_info_2 *info,
-			struct lsh_object *a,
-			struct lsh_object *b)
-{
-  CAST_SUBTYPE(command, callback, a);
-  CAST(io_backend, backend, b);
-  assert(!info->next);
-
-  return &make_listen_with_callback(callback, backend)->super;
-}
-
-static struct collect_info_2 collect_info_listen_callback_2 =
-STATIC_COLLECT_2_FINAL(collect_listen_callback);
-
-struct collect_info_1 listen_with_callback =
-STATIC_COLLECT_1(&collect_info_listen_callback_2);
 
 /* GABA:
    (class
@@ -483,24 +451,32 @@ make_connect_local(struct io_backend *backend)
   return &self->super;
 }
 
-DEFINE_COMMAND_SIMPLE(connect_local_command, a)
+DEFINE_COMMAND(connect_local_command)
+     (struct command *s UNUSED,
+      struct lsh_object *a,
+      struct command_continuation *c,
+      struct exception_handler *e UNUSED)
 {
   CAST(io_backend, backend, a);
-  return &make_connect_local(backend)->super;
+  COMMAND_RETURN(c, make_connect_local(backend));
 }
 
 
 /* Takes a listen_value as argument, logs the peer address, and
  * returns the fd object. */
 
-DEFINE_COMMAND_SIMPLE(io_log_peer_command, a)
+DEFINE_COMMAND(io_log_peer_command)
+     (struct command *s UNUSED,
+      struct lsh_object *a,
+      struct command_continuation *c,
+      struct exception_handler *e UNUSED)
 {
   CAST(listen_value, lv, a);
 
   verbose("Accepting connection from %S, port %i\n",
 	  lv->peer->ip, lv->peer->port);
 
-  return &lv->super;
+  COMMAND_RETURN(c, lv);
 }
 
 
