@@ -2,7 +2,7 @@
  *
  *
  *
- * $Id: keyexchange.h,v 1.50 2001/03/14 18:28:49 nisse Exp $ */
+ * $Id: keyexchange.h,v 1.51 2001/07/06 20:13:10 nisse Exp $ */
 
 /* lsh, an implementation of the ssh protocol
  *
@@ -142,7 +142,7 @@ make_simple_kexinit(struct randomness *r,
 		    struct int_list *compression_algorithms,
 		    struct int_list *languages);
 
-extern struct command_simple kexinit_filter;
+extern struct command_2 kexinit_filter;
 #define KEXINIT_FILTER (&kexinit_filter.super.super)
 
 void initiate_keyexchange(struct ssh_connection *connection);
