@@ -2,7 +2,7 @@
  *
  * client main program
  *
- * $Id: lsh.c,v 1.143 2001/08/21 11:14:44 nisse Exp $ */
+ * $Id: lsh.c,v 1.146 2001/09/01 23:01:28 nisse Exp $ */
 
 /* lsh, an implementation of the ssh protocol
  *
@@ -105,8 +105,6 @@ STATIC_REQUEST_SERVICE(ATOM_SSH_USERAUTH);
      (vars
        (algorithms object algorithms_options)
 
-       (random object randomness_with_poll)
-
        (signature_algorithms object alist)
        (home . "const char *")
        
@@ -138,16 +136,16 @@ make_options(struct io_backend *backend,
 	     int *exit_code)
 {
   NEW(lsh_options, self);
-  init_client_options(&self->super, backend, handler, exit_code);
+  init_client_options(&self->super, backend,
+		      make_default_random(NULL, handler),
+		      handler, exit_code);
 
   self->algorithms
     = make_algorithms_options(all_symmetric_algorithms());
 
-  self->random = make_default_random(NULL, handler);
-  
   self->home = getenv("HOME");
   
-  self->signature_algorithms = all_signature_algorithms(&self->random->super);
+  self->signature_algorithms = all_signature_algorithms(&self->super.random->super);
 
   self->sloppy = 0;
   self->capture = NULL;
@@ -626,10 +624,15 @@ main_options[] =
     "Request the ssh-userauth service (default if SRP is used).", 0 },
 #endif
   
-  /* ACtions */
+  /* Actions */
   { "forward-remote-port", 'R', "remote-port:target-host:target-port",
     0, "", CLIENT_ARGP_ACTION_GROUP },
   { "gateway", 'G', NULL, 0, "Setup a local gateway", 0 },
+
+  { "x11-forward", 'x', NULL, 0, "Enable X11 forwarding.", CLIENT_ARGP_MODIFIER_GROUP },
+  { "no-x11-forward", 'x' | ARG_NOT, NULL, 0,
+    "Disable X11 forwarding (default).", 0 },
+  
   { NULL, 0, NULL, 0, NULL, 0 }
 };
 
@@ -708,7 +711,7 @@ main_argp_parser(int key, char *arg, struct argp_state *state)
 	      LIST(self->kex_algorithms)[i++] = ATOM_SRP_RING1_SHA1_LOCAL;
 	      ALIST_SET(self->algorithms->algorithms,
 			ATOM_SRP_RING1_SHA1_LOCAL,
-			&make_srp_client(make_srp1(&self->random->super),
+			&make_srp_client(make_srp1(&self->super.random->super),
 					 self->super.tty,
 					 ssh_format("%lz", self->super.user))
 			->super);
@@ -719,7 +722,7 @@ main_argp_parser(int key, char *arg, struct argp_state *state)
 	      LIST(self->kex_algorithms)[i++] = ATOM_DIFFIE_HELLMAN_GROUP1_SHA1;
 	      ALIST_SET(self->algorithms->algorithms,
 			ATOM_DIFFIE_HELLMAN_GROUP1_SHA1,
-			&make_dh_client(make_dh1(&self->random->super))
+			&make_dh_client(make_dh1(&self->super.random->super))
 			->super);
 	    }
 	}
@@ -799,7 +802,7 @@ main_argp_parser(int key, char *arg, struct argp_state *state)
 	}
       
       /* Start background poll */
-      RANDOM_POLL_BACKGROUND(self->random->poller);
+      RANDOM_POLL_BACKGROUND(self->super.random->poller);
       
       break;
       
@@ -849,6 +852,7 @@ main_argp_parser(int key, char *arg, struct argp_state *state)
       }      
       
     CASE_FLAG('G', start_gateway);
+    CASE_FLAG('x', super.with_x11);
     }
 
   return 0;
@@ -955,10 +959,10 @@ int main(int argc, char **argv)
 	make_handshake_info(CONNECTION_CLIENT,
 			    "lsh - a free ssh", NULL,
 			    SSH_MAX_PACKET,
-			    &options->random->super,
+			    &options->super.random->super,
 			    options->algorithms->algorithms,
 			    NULL),
-	make_simple_kexinit(&options->random->super,
+	make_simple_kexinit(&options->super.random->super,
 			    options->kex_algorithms,
 			    options->algorithms->hostkey_algorithms,
 			    options->algorithms->crypto_algorithms,
