@@ -1,6 +1,6 @@
 /* lsh.h
  *
- * $Id: lsh.h,v 1.20 2001/04/08 15:13:54 nisse Exp $ */
+ * $Id: lsh.h,v 1.21 2001/09/01 10:38:54 nisse Exp $ */
 
 /* lsh, an implementation of the ssh protocol
  *
@@ -106,6 +106,9 @@ struct lsh_string
 
 /* abstract_crypto.h */
 
+/* client_x11.c */
+struct client_x11_display;
+
 /* connection.h */
 struct ssh_connection;
 
