@@ -1,6 +1,6 @@
 /* lshg.c
  *
- * $Id: lshg.c,v 1.12 2001/08/12 20:27:00 nisse Exp $
+ * $Id: lshg.c,v 1.13 2001/09/01 23:02:09 nisse Exp $
  *
  * Connect to a gateway. */
 
@@ -63,7 +63,7 @@ make_options(struct io_backend *backend,
 	     int *exit_code) 
 {
   NEW(lshg_options, self);
-  init_client_options(&self->super, backend, handler, exit_code);
+  init_client_options(&self->super, backend, NULL, handler, exit_code);
 
   self->gateway = NULL;
 
