@@ -4,7 +4,7 @@
  * processes and ports. Used to kill or release the resource in
  * question when the connection dies.
  *
- * $Id: resource.c,v 1.17 2001/04/17 19:44:00 nisse Exp $ */
+ * $Id: resource.c,v 1.18 2001/09/01 10:43:38 nisse Exp $ */
 
 /* lsh, an implementation of the ssh protocol
  *
@@ -80,7 +80,7 @@ do_resource_kill(struct resource *self)
 { self->alive = 0; }
 
 void
-resource_init(struct resource *self,
+init_resource(struct resource *self,
 	      void (*k)(struct resource *))
 {
   self->alive = 1;
@@ -188,7 +188,7 @@ struct resource_list *
 empty_resource_list(void)
 {
   NEW(concrete_resource_list, self);
-  resource_init(&self->super.super, do_kill_all);
+  init_resource(&self->super.super, do_kill_all);
 
   trace("empty_resource_list: created %xi\n", self);
   self->super.remember = do_remember_resource;
