@@ -1,6 +1,6 @@
 /* tcpforward_commands.c
  *
- * $Id: tcpforward_commands.c,v 1.33 2001/08/28 18:33:08 nisse Exp $
+ * $Id: tcpforward_commands.c,v 1.35 2001/09/01 19:02:55 nisse Exp $
  */
 
 /* lsh, an implementation of the ssh protocol
@@ -108,7 +108,7 @@ do_tcpip_start_io(struct command *s UNUSED,
 		  struct command_continuation *c,
 		  struct exception_handler *e UNUSED)
 {
-  CAST_SUBTYPE(ssh_channel, channel, x);
+  CAST(channel_forward, channel, x);
 
   assert(channel);
   
@@ -159,8 +159,8 @@ new_tcpip_channel(struct channel_open_command *c,
 
   debug("tcpforward_commands.c: new_tcpip_channel\n");
 
-  channel = make_channel_forward(self->peer->fd, TCPIP_WINDOW_SIZE);
-  channel->write = connection->write;
+  channel = &make_channel_forward(self->peer->fd, TCPIP_WINDOW_SIZE)->super;
+  channel->connection = connection;
 
   *request = format_channel_open(self->type, local_channel_number,
 				 channel, 
