@@ -25,29 +25,37 @@
 
 
 
+
+
+
 
 
 
 
 #include "lsh.h"
 
+#include "algorithms.h"
 #include "crypto.h"
 #include "digits.h"
 #include "format.h"
+#include "randomness.h"
 #include "sexp.h"
 #include "spki.h"
+#include "werror.h"
 #include "xalloc.h"
 
 #include <assert.h>
 #include <stdio.h>
+#include <string.h>
 
-int main(int argc UNUSED, char **argv UNUSED)
+int main(int argc, char **argv)
 {
+  argp_parse(&werror_argp, argc, argv, 0, NULL, NULL);
 /* Test vectors from RFC 1321 */
 
   {
     struct lsh_string *a, *b;
-    fputs("MD5-1... ", stderr);
+    fputs("MD5-1 ... ", stderr);
     a = hash_string(&md5_algorithm, ssh_format("%lz", "") , 1);
     b = simple_decode_hex("D41D8CD98F00B204E9800998ECF8427E") ;
     if (!lsh_string_eq(a, b))
@@ -61,7 +69,7 @@ int main(int argc UNUSED, char **argv UNUSED)
 
   {
     struct lsh_string *a, *b;
-    fputs("MD5-2... ", stderr);
+    fputs("MD5-2 ... ", stderr);
     a = hash_string(&md5_algorithm, ssh_format("%lz", "a") , 1);
     b = simple_decode_hex("0CC175B9C0F1B6A831C399E269772661") ;
     if (!lsh_string_eq(a, b))
@@ -75,7 +83,7 @@ int main(int argc UNUSED, char **argv UNUSED)
 
   {
     struct lsh_string *a, *b;
-    fputs("MD5-3... ", stderr);
+    fputs("MD5-3 ... ", stderr);
     a = hash_string(&md5_algorithm, ssh_format("%lz", "abc") , 1);
     b = simple_decode_hex("900150983cd24fb0D6963F7D28E17F72") ;
     if (!lsh_string_eq(a, b))
@@ -89,7 +97,7 @@ int main(int argc UNUSED, char **argv UNUSED)
 
   {
     struct lsh_string *a, *b;
-    fputs("MD5-4... ", stderr);
+    fputs("MD5-4 ... ", stderr);
     a = hash_string(&md5_algorithm, ssh_format("%lz", "message digest") , 1);
     b = simple_decode_hex("F96B697D7CB7938D525A2F31AAF161D0") ;
     if (!lsh_string_eq(a, b))
@@ -103,7 +111,7 @@ int main(int argc UNUSED, char **argv UNUSED)
 
   {
     struct lsh_string *a, *b;
-    fputs("MD5-5... ", stderr);
+    fputs("MD5-5 ... ", stderr);
     a = hash_string(&md5_algorithm, ssh_format("%lz", "abcdefghijklmnopqrstuvwxyz") , 1);
     b = simple_decode_hex("C3FCD3D76192E4007DFB496CCA67E13B") ;
     if (!lsh_string_eq(a, b))
@@ -117,7 +125,7 @@ int main(int argc UNUSED, char **argv UNUSED)
 
   {
     struct lsh_string *a, *b;
-    fputs("MD5-6... ", stderr);
+    fputs("MD5-6 ... ", stderr);
     a = hash_string(&md5_algorithm, ssh_format("%lz", "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789") , 1);
     b = simple_decode_hex("D174AB98D277D9F5A5611C2C9F419D9F") ;
     if (!lsh_string_eq(a, b))
@@ -131,7 +139,7 @@ int main(int argc UNUSED, char **argv UNUSED)
 
   {
     struct lsh_string *a, *b;
-    fputs("MD5-7... ", stderr);
+    fputs("MD5-7 ... ", stderr);
     a = hash_string(&md5_algorithm, ssh_format("%lz", "1234567890123456789012345678901234567890"
              "1234567890123456789012345678901234567890") , 1);
     b = simple_decode_hex("57EDF4A22BE3C955AC49DA2E2107B67A") ;
@@ -152,7 +160,7 @@ int main(int argc UNUSED, char **argv UNUSED)
     
   {
     struct lsh_string *a, *b;
-    fputs("HMAC-MD5-1... ", stderr);
+    fputs("HMAC-MD5-1 ... ", stderr);
     a = mac_string(hmac, key, 1,
                                        ssh_format("%lz", "Hi There") , 1);
     b = simple_decode_hex("9294727A3638BB1C13F48EF8158BFC9D") ;
@@ -174,7 +182,7 @@ int main(int argc UNUSED, char **argv UNUSED)
     
   {
     struct lsh_string *a, *b;
-    fputs("HMAC-MD5-2... ", stderr);
+    fputs("HMAC-MD5-2 ... ", stderr);
     a = mac_string(hmac, key, 1,
                                        ssh_format("%lz", "what do ya want for nothing?") , 1);
     b = simple_decode_hex("750C783E6AB0B503EAA86E310A5DB738") ;
@@ -196,7 +204,7 @@ int main(int argc UNUSED, char **argv UNUSED)
     
   {
     struct lsh_string *a, *b;
-    fputs("HMAC-MD5-3... ", stderr);
+    fputs("HMAC-MD5-3 ... ", stderr);
     a = mac_string(hmac, key, 1,
                                        simple_decode_hex("DDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDD") , 1);
     b = simple_decode_hex("56BE34521D144C88DBB8C733F0E8B3F6") ;
@@ -218,7 +226,7 @@ int main(int argc UNUSED, char **argv UNUSED)
     
   {
     struct lsh_string *a, *b;
-    fputs("HMAC-MD5-4... ", stderr);
+    fputs("HMAC-MD5-4 ... ", stderr);
     a = mac_string(hmac, key, 1,
                                        simple_decode_hex("CDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCD") , 1);
     b = simple_decode_hex("697EAF0ACA3A3AEA3A75164746FFAA79") ;
@@ -240,7 +248,7 @@ int main(int argc UNUSED, char **argv UNUSED)
     
   {
     struct lsh_string *a, *b;
-    fputs("HMAC-MD5-5... ", stderr);
+    fputs("HMAC-MD5-5 ... ", stderr);
     a = mac_string(hmac, key, 1,
                                        ssh_format("%lz", "Test With Truncation") , 1);
     b = simple_decode_hex("56461EF2342EDC00F9BAB995690EFD4C") ;
@@ -262,7 +270,7 @@ int main(int argc UNUSED, char **argv UNUSED)
     
   {
     struct lsh_string *a, *b;
-    fputs("HMAC-MD5-6... ", stderr);
+    fputs("HMAC-MD5-6 ... ", stderr);
     a = mac_string(hmac, key, 1,
                                        ssh_format("%lz", "Test Using Larger Than Block-Size Key - Hash Key First") , 1);
     b = simple_decode_hex("6b1ab7fe4bd7bf8f0b62e6ce61b9d0cd") ;
@@ -284,7 +292,7 @@ int main(int argc UNUSED, char **argv UNUSED)
     
   {
     struct lsh_string *a, *b;
-    fputs("HMAC-MD5-7... ", stderr);
+    fputs("HMAC-MD5-7 ... ", stderr);
     a = mac_string(hmac, key, 1,
                                        ssh_format("%lz", "Test Using Larger Than Block-Size Key an"
              "d Larger Than One Block-Size Data") , 1);
