@@ -2,7 +2,7 @@
  *
  * Interact with the user.
  *
- * $Id: unix_interact.c,v 1.8 2001/06/19 21:10:01 nisse Exp $ */
+ * $Id: unix_interact.c,v 1.9 2001/09/01 10:43:39 nisse Exp $ */
 
 /* lsh, an implementation of the ssh protocol
  *
@@ -304,7 +304,7 @@ unix_window_change_subscribe(struct interact *s,
   CAST(unix_interact, self, s);
 
   NEW(window_subscriber, subscriber);
-  resource_init(&subscriber->super, NULL);
+  init_resource(&subscriber->super, NULL);
 
   subscriber->next = self->subscribers;
   subscriber->callback = callback;
