@@ -2,7 +2,7 @@
  *
  * User-related functions on UN*X
  *
- * $Id: unix_user.c,v 1.39 2001/05/29 21:10:52 nisse Exp $
+ * $Id: unix_user.c,v 1.40 2001/09/01 10:43:39 nisse Exp $
  */
 
 /* lsh, an implementation of the ssh protocol
@@ -224,7 +224,7 @@ static struct lsh_process *
 make_process_resource(pid_t pid, int signal)
 {
   NEW(process_resource, self);
-  resource_init(&self->super.super, do_kill_process);
+  init_resource(&self->super.super, do_kill_process);
   self->super.signal = do_signal_process;
   
   self->pid = pid;
