@@ -2,7 +2,7 @@
  *
  *
  *
- * $Id: write_buffer.c,v 1.20 1999/08/30 19:31:10 nisse Exp $ */
+ * $Id: write_buffer.c,v 1.22 2001/04/04 21:08:10 nisse Exp $ */
 
 /* lsh, an implementation of the ssh protocol
  *
@@ -31,28 +31,16 @@
 #include <assert.h>
 #include <string.h>
 
-/* Prototype */
-static void do_free_buffer(struct lsh_queue *q);
-
 #define GABA_DEFINE
 #include "write_buffer.h.x"
 #undef GABA_DEFINE
 
-static void do_free_buffer(struct lsh_queue *q)
-{
-  FOR_QUEUE(q, struct buffer_node *, n)
-    {
-      lsh_string_free(n->packet);
-      lsh_space_free(n);
-    }
-}
 
 static void
 do_write(struct abstract_write *w,
 	 struct lsh_string *packet)
 {
   CAST(write_buffer, closure, w);
-  struct buffer_node *new;
 
   debug("write_buffer: do_write length = %i\n",
 	packet->length);
@@ -70,10 +58,8 @@ do_write(struct abstract_write *w,
     }
   
   /* Enqueue packet */
-  NEW_SPACE(new);
-  new->packet = packet;
 
-  lsh_queue_add_tail(&closure->q, &new->header);
+  string_queue_add_tail(&closure->q, packet);
   
 #if 0
   if (closure->try_write)
@@ -146,15 +132,10 @@ int write_buffer_pre_write(struct write_buffer *buffer)
       else
 	{
 	  /* Dequeue a packet, if possible */
-	  if (!lsh_queue_is_empty(&buffer->q))
-	    {	    
-	      struct buffer_node *n =
-		(struct buffer_node *) lsh_queue_remove_head(&buffer->q);
-	      
-	      buffer->partial = n->packet;
+	  if (!string_queue_is_empty(&buffer->q))
+	    {
+	      buffer->partial = string_queue_remove_head(&buffer->q);
 	      buffer->pos = 0;
-
-	      lsh_space_free(n);
 	    }
 	  else
 	    break;
@@ -179,7 +160,8 @@ void write_buffer_close(struct write_buffer *buffer)
   buffer->closed = 1;
 }
 
-struct write_buffer *write_buffer_alloc(UINT32 size)
+struct write_buffer *
+make_write_buffer(UINT32 size)
 {
   NEW(write_buffer, res);
   
@@ -198,7 +180,7 @@ struct write_buffer *write_buffer_alloc(UINT32 size)
   res->try_write = try; 
 #endif
   
-  lsh_queue_init(&res->q);
+  string_queue_init(&res->q);
 
   res->pos = 0;
   res->partial = NULL;
